'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.update = exports.destroyScope = exports.destroy = exports.scopeScope = exports.scope = exports.list = exports.get = undefined;

var _joi = require('joi');

var _joi2 = _interopRequireDefault(_joi);

var _error = require('../error');

var _error2 = _interopRequireDefault(_error);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) {
  var desc = {};
  Object['ke' + 'ys'](descriptor).forEach(function (key) {
    desc[key] = descriptor[key];
  });
  desc.enumerable = !!desc.enumerable;
  desc.configurable = !!desc.configurable;

  if ('value' in desc || desc.initializer) {
    desc.writable = true;
  }

  desc = decorators.slice().reverse().reduce(function (desc, decorator) {
    return decorator(target, property, desc) || desc;
  }, desc);

  if (context && desc.initializer !== void 0) {
    desc.value = desc.initializer ? desc.initializer.call(context) : void 0;
    desc.initializer = undefined;
  }

  if (desc.initializer === void 0) {
    Object['define' + 'Property'](target, property, desc);
    desc = null;
  }

  return desc;
}

let prefix;
let defaultConfig;

exports.default = (server, a, b, names, options) => {
  prefix = options.prefix;
  defaultConfig = options.defaultConfig;

  get(server, a, b, names);
  list(server, a, b, names);
  scope(server, a, b, names);
  scopeScope(server, a, b, names);
  destroy(server, a, b, names);
  destroyScope(server, a, b, names);
  update(server, a, b, names);
};

const get = exports.get = (server, a, b, names) => {
  var _desc, _value, _obj;

  server.route((_obj = {
    method: 'GET',
    path: `${ prefix }/${ names.a.singular }/{aid}/${ names.b.singular }/{bid}`,

    handler(request, reply) {
      return _asyncToGenerator(function* () {
        const include = (0, _utils.parseInclude)(request);

        const base = yield a.findOne({
          where: {
            [a.primaryKeyField]: request.params.aid
          }
        });

        const method = (0, _utils.getMethod)(base, names.b);

        const list = yield method({ where: {
            [b.primaryKeyField]: request.params.bid
          }, include });

        if (Array.isArray(list)) {
          reply(list[0]);
        } else {
          reply(list);
        }
      })();
    },

    config: defaultConfig
  }, (_applyDecoratedDescriptor(_obj, 'handler', [_error2.default], Object.getOwnPropertyDescriptor(_obj, 'handler'), _obj)), _obj));
};

const list = exports.list = (server, a, b, names) => {
  var _desc2, _value2, _obj2;

  server.route((_obj2 = {
    method: 'GET',
    path: `${ prefix }/${ names.a.singular }/{aid}/${ names.b.plural }`,

    handler(request, reply) {
      return _asyncToGenerator(function* () {
        const include = (0, _utils.parseInclude)(request);
        const where = (0, _utils.parseWhere)(request);

        const base = yield a.findOne({
          where: {
            [a.primaryKeyField]: request.params.aid
          }
        });

        const method = (0, _utils.getMethod)(base, names.b);
        const list = yield method({ where, include });

        reply(list);
      })();
    },

    config: defaultConfig
  }, (_applyDecoratedDescriptor(_obj2, 'handler', [_error2.default], Object.getOwnPropertyDescriptor(_obj2, 'handler'), _obj2)), _obj2));
};

const scope = exports.scope = (server, a, b, names) => {
  var _desc3, _value3, _obj3;

  const scopes = Object.keys(b.options.scopes);

  server.route((_obj3 = {
    method: 'GET',
    path: `${ prefix }/${ names.a.singular }/{aid}/${ names.b.plural }/{scope}`,

    handler(request, reply) {
      return _asyncToGenerator(function* () {
        const include = (0, _utils.parseInclude)(request);
        const where = (0, _utils.parseWhere)(request);

        const base = yield a.findOne({
          where: {
            [a.primaryKeyField]: request.params.aid
          }
        });

        const method = (0, _utils.getMethod)(base, names.b);
        const list = yield method({
          scope: request.params.scope,
          where,
          include
        });

        reply(list);
      })();
    },

    config: _lodash2.default.defaultsDeep({
      validate: {
        params: _joi2.default.object().keys({
          scope: _joi2.default.string().valid(...scopes),
          aid: _joi2.default.number().integer().required()
        })
      }
    }, defaultConfig)
  }, (_applyDecoratedDescriptor(_obj3, 'handler', [_error2.default], Object.getOwnPropertyDescriptor(_obj3, 'handler'), _obj3)), _obj3));
};

const scopeScope = exports.scopeScope = (server, a, b, names) => {
  var _desc4, _value4, _obj4;

  const scopes = {
    a: Object.keys(a.options.scopes),
    b: Object.keys(b.options.scopes)
  };

  server.route((_obj4 = {
    method: 'GET',
    path: `${ prefix }/${ names.a.plural }/{scopea}/${ names.b.plural }/{scopeb}`,

    handler(request, reply) {
      return _asyncToGenerator(function* () {
        const include = (0, _utils.parseInclude)(request);
        const where = (0, _utils.parseWhere)(request);

        const list = yield b.scope(request.params.scopeb).findAll({
          where,
          include: include.concat({
            model: a.scope(request.params.scopea)
          })
        });

        reply(list);
      })();
    },

    config: _lodash2.default.defaultsDeep({
      validate: {
        params: _joi2.default.object().keys({
          scopea: _joi2.default.string().valid(...scopes.a),
          scopeb: _joi2.default.string().valid(...scopes.b)
        })
      }
    }, defaultConfig)
  }, (_applyDecoratedDescriptor(_obj4, 'handler', [_error2.default], Object.getOwnPropertyDescriptor(_obj4, 'handler'), _obj4)), _obj4));
};

function _ref(item) {
  return item.destroy();
}

const destroy = exports.destroy = (server, a, b, names) => {
  var _desc5, _value5, _obj5;

  server.route((_obj5 = {
    method: 'DELETE',
    path: `${ prefix }/${ names.a.singular }/{aid}/${ names.b.plural }`,

    handler(request, reply) {
      return _asyncToGenerator(function* () {
        const include = (0, _utils.parseInclude)(request);
        const where = (0, _utils.parseWhere)(request);

        const base = yield a.findOne({
          where: {
            [a.primaryKeyField]: request.params.aid
          }
        });

        const method = (0, _utils.getMethod)(base, names.b, true, 'get');
        const list = yield method({ where, include });
        yield Promise.all(list.map(_ref));

        reply(list);
      })();
    }
  }, (_applyDecoratedDescriptor(_obj5, 'handler', [_error2.default], Object.getOwnPropertyDescriptor(_obj5, 'handler'), _obj5)), _obj5));
};

function _ref2(instance) {
  return instance.destroy();
}

const destroyScope = exports.destroyScope = (server, a, b, names) => {
  var _desc6, _value6, _obj6;

  const scopes = Object.keys(b.options.scopes);

  server.route((_obj6 = {
    method: 'DELETE',
    path: `${ prefix }/${ names.a.singular }/{aid}/${ names.b.plural }/{scope}`,

    handler(request, reply) {
      return _asyncToGenerator(function* () {
        const include = (0, _utils.parseInclude)(request);
        const where = (0, _utils.parseWhere)(request);

        const base = yield a.findOne({
          where: {
            [a.primarykeyField]: request.params.aid
          }
        });

        const method = (0, _utils.getMethod)(base, names.b, true, 'get');

        const list = yield method({
          scope: request.params.scope,
          where,
          include
        });

        yield Promise.all(list.map(_ref2));

        reply(list);
      })();
    },

    config: _lodash2.default.defaultsDeep({
      validate: {
        params: _joi2.default.object().keys({
          scope: _joi2.default.string().valid(...scopes),
          aid: _joi2.default.number().integer().required()
        })
      }
    }, defaultConfig)
  }, (_applyDecoratedDescriptor(_obj6, 'handler', [_error2.default], Object.getOwnPropertyDescriptor(_obj6, 'handler'), _obj6)), _obj6));
};

const update = exports.update = (server, a, b, names) => {
  var _desc7, _value7, _obj7;

  server.route((_obj7 = {
    method: 'PUT',
    path: `${ prefix }/${ names.a.singular }/{aid}/${ names.b.plural }`,

    handler(request, reply) {
      function _ref3(instance) {
        return instance.update(request.payload);
      }

      return _asyncToGenerator(function* () {
        const include = (0, _utils.parseInclude)(request);
        const where = (0, _utils.parseWhere)(request);

        const base = yield a.findOne({
          where: {
            [a.primaryKeyField]: request.params.aid
          }
        });

        const method = (0, _utils.getMethod)(base, names.b);
        const list = yield method({ where, include });

        yield Promise.all(list.map(_ref3));

        reply(list);
      })();
    },

    config: defaultConfig
  }, (_applyDecoratedDescriptor(_obj7, 'handler', [_error2.default], Object.getOwnPropertyDescriptor(_obj7, 'handler'), _obj7)), _obj7));
};