'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.associations = exports.update = exports.destroyScope = exports.destroyAll = exports.destroy = exports.create = exports.scope = exports.get = exports.list = undefined;

var _joi = require('joi');

var _joi2 = _interopRequireDefault(_joi);

var _error = require('./error');

var _error2 = _interopRequireDefault(_error);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _utils = require('./utils');

var _boom = require('boom');

var _index = require('./associations/index');

var associations = _interopRequireWildcard(_index);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) {
  var desc = {};
  Object['ke' + 'ys'](descriptor).forEach(function (key) {
    desc[key] = descriptor[key];
  });
  desc.enumerable = !!desc.enumerable;
  desc.configurable = !!desc.configurable;

  if ('value' in desc || desc.initializer) {
    desc.writable = true;
  }

  desc = decorators.slice().reverse().reduce(function (desc, decorator) {
    return decorator(target, property, desc) || desc;
  }, desc);

  if (context && desc.initializer !== void 0) {
    desc.value = desc.initializer ? desc.initializer.call(context) : void 0;
    desc.initializer = undefined;
  }

  if (desc.initializer === void 0) {
    Object['define' + 'Property'](target, property, desc);
    desc = null;
  }

  return desc;
}

const createAll = ({ server, model, prefix, config }) => {
  Object.keys(methods).forEach(method => {
    methods[method]({ server, model, prefix, config });
  });
};

/*
The `models` option, becomes `permissions`, and can look like:

```
models: ['cat', 'dog']
```

or

```
models: {
  cat: ['list', 'get']
  , dog: true // all
}
```

*/

exports.default = (server, model, { prefix, defaultConfig: config, models: permissions }) => {
  const modelName = model._singular;
  const permissionsIsArray = Array.isArray(permissions);

  function _ref(permission) {
    return permission.model === modelName;
  }

  function _ref2(permissionOption) {
    if (_lodash2.default.isPlainObject(permissionOption)) {
      const permissionConfig = permissionOption.config || config;

      if (permissionOption.methods) {
        permissionOption.methods.forEach(method => {
          methods[method]({
            server,
            model,
            prefix,
            config: permissionConfig
          });
        });
      } else {
        createAll({ server, model, prefix, config: permissionConfig });
      }
    }
  }

  if (!permissions) {
    createAll({ server, model, prefix, config });
  } else if (permissionsIsArray && permissions.includes(modelName)) {
    createAll({ server, model, prefix, config });
  } else if (permissionsIsArray) {
    const permissionOptions = permissions.filter(_ref);

    permissionOptions.forEach(_ref2);
  }
};

const list = exports.list = ({ server, model, prefix, config }) => {
  var _desc, _value, _obj;

  server.route((_obj = {
    method: 'GET',
    path: `${ prefix }/${ model._plural }`,

    handler(request, reply) {
      return _asyncToGenerator(function* () {
        const include = (0, _utils.parseInclude)(request);
        const where = (0, _utils.parseWhere)(request);

        if (include instanceof Error) return void reply(include);

        const list = yield model.findAll({
          where, include
        });

        reply(list);
      })();
    },

    config
  }, (_applyDecoratedDescriptor(_obj, 'handler', [_error2.default], Object.getOwnPropertyDescriptor(_obj, 'handler'), _obj)), _obj));
};

const get = exports.get = ({ server, model, prefix, config }) => {
  var _desc2, _value2, _obj2;

  server.route((_obj2 = {
    method: 'GET',
    path: `${ prefix }/${ model._singular }/{id?}`,

    handler(request, reply) {
      return _asyncToGenerator(function* () {
        const include = (0, _utils.parseInclude)(request);
        const where = (0, _utils.parseWhere)(request);
        const { id } = request.params;
        if (id) where[model.primaryKeyField] = id;

        const instance = yield model.findOne({ where, include });

        if (!instance) return void reply((0, _boom.notFound)(`${ id } not found.`));

        reply(instance);
      })();
    },
    config: _lodash2.default.defaultsDeep({
      validate: {
        params: _joi2.default.object().keys({
          id: _joi2.default.any()
        })
      }
    }, config)
  }, (_applyDecoratedDescriptor(_obj2, 'handler', [_error2.default], Object.getOwnPropertyDescriptor(_obj2, 'handler'), _obj2)), _obj2));
};

const scope = exports.scope = ({ server, model, prefix, config }) => {
  var _desc3, _value3, _obj3;

  const scopes = Object.keys(model.options.scopes);

  server.route((_obj3 = {
    method: 'GET',
    path: `${ prefix }/${ model._plural }/{scope}`,

    handler(request, reply) {
      return _asyncToGenerator(function* () {
        const include = (0, _utils.parseInclude)(request);
        const where = (0, _utils.parseWhere)(request);

        const list = yield model.scope(request.params.scope).findAll({ include, where });

        reply(list);
      })();
    },
    config: _lodash2.default.defaultsDeep({
      validate: {
        params: _joi2.default.object().keys({
          scope: _joi2.default.string().valid(...scopes)
        })
      }
    }, config)
  }, (_applyDecoratedDescriptor(_obj3, 'handler', [_error2.default], Object.getOwnPropertyDescriptor(_obj3, 'handler'), _obj3)), _obj3));
};

const create = exports.create = ({ server, model, prefix, config }) => {
  var _desc4, _value4, _obj4;

  server.route((_obj4 = {
    method: 'POST',
    path: `${ prefix }/${ model._singular }`,

    handler(request, reply) {
      return _asyncToGenerator(function* () {
        const instance = yield model.create(request.payload);

        reply(instance);
      })();
    },

    config
  }, (_applyDecoratedDescriptor(_obj4, 'handler', [_error2.default], Object.getOwnPropertyDescriptor(_obj4, 'handler'), _obj4)), _obj4));
};

function _ref3(instance) {
  return instance.destroy();
}

const destroy = exports.destroy = ({ server, model, prefix, config }) => {
  var _desc5, _value5, _obj5;

  server.route((_obj5 = {
    method: 'DELETE',
    path: `${ prefix }/${ model._singular }/{id?}`,

    handler(request, reply) {
      return _asyncToGenerator(function* () {
        const where = (0, _utils.parseWhere)(request);
        if (request.params.id) where[model.primaryKeyField] = request.params.id;

        const list = yield model.findAll({ where });

        yield Promise.all(list.map(_ref3));

        reply(list.length === 1 ? list[0] : list);
      })();
    },

    config
  }, (_applyDecoratedDescriptor(_obj5, 'handler', [_error2.default], Object.getOwnPropertyDescriptor(_obj5, 'handler'), _obj5)), _obj5));
};

function _ref4(instance) {
  return instance.destroy();
}

const destroyAll = exports.destroyAll = ({ server, model, prefix, config }) => {
  var _desc6, _value6, _obj6;

  server.route((_obj6 = {
    method: 'DELETE',
    path: `${ prefix }/${ model._plural }`,

    handler(request, reply) {
      return _asyncToGenerator(function* () {
        const where = (0, _utils.parseWhere)(request);

        const list = yield model.findAll({ where });

        yield Promise.all(list.map(_ref4));

        reply(list.length === 1 ? list[0] : list);
      })();
    },

    config
  }, (_applyDecoratedDescriptor(_obj6, 'handler', [_error2.default], Object.getOwnPropertyDescriptor(_obj6, 'handler'), _obj6)), _obj6));
};

function _ref5(instance) {
  return instance.destroy();
}

const destroyScope = exports.destroyScope = ({ server, model, prefix, config }) => {
  var _desc7, _value7, _obj7;

  const scopes = Object.keys(model.options.scopes);

  server.route((_obj7 = {
    method: 'DELETE',
    path: `${ prefix }/${ model._plural }/{scope}`,

    handler(request, reply) {
      return _asyncToGenerator(function* () {
        const include = (0, _utils.parseInclude)(request);
        const where = (0, _utils.parseWhere)(request);

        const list = yield model.scope(request.params.scope).findAll({ include, where });

        yield Promise.all(list.map(_ref5));

        reply(list);
      })();
    },
    config: _lodash2.default.defaultsDeep({
      validate: {
        params: _joi2.default.object().keys({
          scope: _joi2.default.string().valid(...scopes)
        })
      }
    }, config)
  }, (_applyDecoratedDescriptor(_obj7, 'handler', [_error2.default], Object.getOwnPropertyDescriptor(_obj7, 'handler'), _obj7)), _obj7));
};

const update = exports.update = ({ server, model, prefix, config }) => {
  var _desc8, _value8, _obj8;

  server.route((_obj8 = {
    method: 'PUT',
    path: `${ prefix }/${ model._singular }/{id}`,

    handler(request, reply) {
      return _asyncToGenerator(function* () {
        const { id } = request.params;
        const instance = yield model.findOne({
          where: {
            id
          }
        });

        if (!instance) return void reply((0, _boom.notFound)(`${ id } not found.`));

        yield instance.update(request.payload);

        reply(instance);
      })();
    },

    config: _lodash2.default.defaultsDeep({
      validate: {
        payload: _joi2.default.object().required()
      }
    }, config)
  }, (_applyDecoratedDescriptor(_obj8, 'handler', [_error2.default], Object.getOwnPropertyDescriptor(_obj8, 'handler'), _obj8)), _obj8));
};

exports.associations = associations;


const methods = {
  list, get, scope, create, destroy, destroyAll, destroyScope, update
};