'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getMethod = exports.parseWhere = exports.parseInclude = undefined;

var _lodash = require('lodash');

const parseInclude = exports.parseInclude = request => {
  const include = Array.isArray(request.query.include) ? request.query.include : [request.query.include];

  const noGetDb = typeof request.getDb !== 'function';
  const noRequestModels = !request.models;

  if (noGetDb && noRequestModels) {
    return new Error('`request.getDb` or `request.models` are not defined.' + 'Be sure to load hapi-sequelize before hapi-sequelize-crud.');
  }

  const { models } = noGetDb ? request : request.getDb();

  return include.map(a => {
    if (typeof a === 'string') return models[a];

    if (a && typeof a.model === 'string' && a.model.length) {
      a.model = models[a.model];
    }

    return a;
  }).filter(_lodash.identity);
};

const parseWhere = exports.parseWhere = request => {
  const where = (0, _lodash.omit)(request.query, 'include');

  for (const key of Object.keys(where)) {
    try {
      where[key] = JSON.parse(where[key]);
    } catch (e) {
      //
    }
  }

  return where;
};

const getMethod = exports.getMethod = (model, association, plural = true, method = 'get') => {
  const a = plural ? association.original.plural : association.original.singular;
  const b = plural ? association.original.singular : association.original.plural; // alternative
  const fn = model[`${ method }${ a }`] || model[`${ method }${ b }`];
  if (fn) return fn.bind(model);

  return false;
};